/*
 * LaneEmden.h - Definicion de clase LaneEmden
 *
 *    Copyright (C) 2012 - 2015  Miguel Molina
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/**
 * @file LaneEmden.h
 * @brief Definición de clase LaneEmden
 * @details Definición de clase con sus respectivos miembros para
 * para resolver la ecuación diferencial de Lane-Emden por el método
 * de Runge-Kutta.
 */

#ifndef __LANE_EMDEN_H_
#define __LANE_EMDEN_H_

#include<iostream>

using namespace std;

#define N 2              /**< Dimension vectorial del problema diferencial */
#define S 7              /**< Orden del Metodo Runge-Kutta */

/**
 * @class LaneEmden
 * @brief Clase función Lane-Emden
 * @details Implementación de funciones que resuelven numéricamente la
 * ecuación de Lane-Emden; el método numérico elegido es el método de
 * Runge-Kutta de orden 7.
 */
class LaneEmden{
private:
  int sgn;
  double t, x, y, n;                                /**< Variables independientes y dependientes */
  double R[N];                                      /**< Vector solución de N dimensiones */
  double A[S][S], B[S], C[S];                       /**< Parámetros del método Runge-Kutta */
public:
  /**
   * @brief Función matriz de Butcher
   * @details Definición de coeficientes para método Runge-Kutta
   * (explicito) de sexto orden.
   */
  void MatrizButcher(void);
  /**
   * @brief Método Runge-Kutta orden S
   * @param h  Paso de integración
   * @details Proceso iterativo del método Runge-Kutta de orden
   * <i>S</i>=6.
   */
  void RungeKuttaS(double h);
  /**
   * @brief Función ecuación de Lane-Emden
   * @param tn  Variable independiente
   * @param Rn  Vector solución
   * @param Fn  Vector dinámico
   * @details Ecuación diferencial de Lane-Emden de índice \f$n\f$.
   * La ecuación diferencial de segundo orden es reducida a un sistema
   * de dos ecuaciones de primer orden con \f$R(t)=\{x(t),y(t)\}\f$,
   * donde \f$x(t)=\theta(t)\f$, \f$y(t)=\theta'(t)\f$.
   */
  void Ecuacion(double tn,double *Rn,double *Fn);
  /**
   * @brief Función inicial
   * @param n0  Índice de la función \f$\theta_n(t)\f$
   * @details Condiciones iniciales para la ecuación diferencial de
   * Lane-Emden de índice \f$n\f$, con \f$R(t)=\{1,0\}\f$. Para evitar
   * la indeterminación en \f$t=0\f$ se usa la regla L'Hopital en
   * \f$y'(t)\f$.
   */
  void inicio(double n0);
  /**
   * @brief Función de actualización
   * @param tn  Variable independiente
   * @details Función auxiliar para actualizar el vector solución
   * \f$R\f$ en cada iteración.
   */
  void updateRn(double tn);
  /**
   * @return Impresión variable independiente
   */
  double imp_t(void){return t;};
  /**
   * @return Impresión de la función \f$\theta_n(t)\f$
   */
  double imp_x(void){return x;};
  /**
   * @return Impresión de la primera derivada \f$\theta_n'(t)\f$
   */
  double imp_y(void){return y;};
  /**
   * @return Bandera de control de signo en la función
   */
  int flag(void){return sgn;};
};

#endif /* __LANE_EMDEN_H_ */
