/*
 * mainSLEn.cpp - Solucion numerica de la ecuacion de Lane-Emden de indice n 
 *
 *    Copyright (C) 2012 - 2015  Miguel Molina
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/**
 * @mainpage Solvelen - Documentación de desarrollo
 *
 * @author Miguel Molina
 *
 * @version 0.1.3
 *
 * @date 2015-04-29
 *
 * @copyright Copyright (C) 2015 Miguel Molina
 *
 * @section license Licencia
 * Licencia GPLv3+: GNU GPL versión 3 o posterior \<http://gnu.org/licenses/gpl.html\>.<br>
 * Este programa se ofrece SIN GARANTÍA ALGUNA. Es software libre, y usted puede<br>
 * redistribuirlo bajo ciertas condiciones.
 *
 * @file mainSLEn.cpp
 * @brief Aplicación e implementación de funciones para salida
 * principal de la interfaz de línea de comandos (CLI) de solvelen
 */

#ifdef HAVE_CONFIG_H
# include<config.h>
# define _AUTOCONF_
#endif

#include<iostream>
#include<cstdlib>
#include<iomanip>
#include<string.h>
#include"nsolveLEn.h"

/// @cond
#ifdef ENABLE_NLS
# include <locale.h>
# include "gettext.h"
# define _(string) (const char *) gettext(string)
#else
# define _(string) (string)
#endif
#define N_(string) string
/// @endcond

#ifdef _AUTOCONF_
# if !defined(STDC_HEADERS)
#  if !HAVE_STRRCHR
#   define strrchr rindex
#  endif
char *strrchr ();
# endif
#endif

#if defined(unix) 
static const char charsepdir='/';
#elif defined(WIN32)
static const char charsepdir='\\';
#elif defined(__APPLE__)
static const char charsepdir='/';
#else
# error "Define something here for your operating system"
#endif

using namespace std;

/// @cond
const double dt=2.5e-4;         // Paso de integracion
const double npi=3.141592653589793; // Numero pi
/// @endcond
static const char * programa;   // Nombre programa
// Opciones cortas y largas
static const char * opc_cortas[] = {"r", "d", "a", "o", "v", "V", "h"};
static const char * opc_largas[] = {"raiz",
				    "derivada",
				    "all",
				    "outfile",
				    "verbose",
				    "version",
				    "help"};
/**
 * @enum TAGOPCION
 * Enumerador para identificar las opciones del programa.
 */
// TAGOPCION: Etiquetas
enum TAGOPCION {RAIZ=1,	DERIVADA, TODO,	ARCHIVO, VERBOSO, VERSNPROG, AYUDA};


/* Mensaje de error */
static void error_std(const char * argerr, const char * msjerr, int msjuso){
  cerr << "Error: '" << argerr << "'; " << msjerr << endl;
  if( msjuso )
    cerr << _("Use '") << programa << _(" -h' for more information.") << endl;
  exit(EXIT_FAILURE);
}

/* Nombre base del programa */
static char *basenameprog(char *path){
  char *ptr = strrchr(path,charsepdir);
  return ptr ? ptr + 1 : (char*)path;
}

/* Verificacion de cadena numerica */
static int verificar_numero(char * cad){
  const char * ref="abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ,:_{}[]+/=^~#$%";
  char * ptr;
  ptr=strpbrk(cad, ref);
  if( ptr || !strcmp(cad,"") )
    return 0;  // No es un numero
  else
    return 1;  // Si es un numero
}

/* Verificacion de opcion */
// Retorna 1 si algo="-o" o si algo="--opcion", retorna 0 en otro caso.
static int verificar_opcion_corta(char * cad, const char * opcc){
  if( strlen(cad)==2 && cad[0]=='-' && cad[1]==opcc[0] )
    return 1;
  else
    return 0;
}

static int verificar_opcion_larga(char * cad, const char * opcl){
  if( !strncmp(cad,"--",2) && !strcmp(cad+2, opcl) )
    return 1;
  else
    return 0;
}

/* Analisis de opcion */
static int obtopc(char * opc){
  int i, opc_num;
  opc_num=0;
  for(i=0; i<7; i++){
    if( verificar_opcion_corta(opc, opc_cortas[i]) ){
      opc_num=i+1;
      break;
    }
    if( verificar_opcion_larga(opc, opc_largas[i]) ){
      opc_num=i+1;
      break;
    }
  }
  if( !opc_num )
    error_std(opc, _("invalid option."), 1);
  return opc_num;
}

/* Verificacion de opciones y argumentos del programa */
static int verificar_arg_prog(int _argc, char ** _argv){
  int i, j, argout, argverb;
  char * tmp;
  argverb=0;
  argout=0;
  j=0;
  tmp='\0';
  for(i=1; i<_argc; i++){
    if( verificar_opcion_corta(_argv[i], opc_cortas[4]) || verificar_opcion_larga(_argv[i], opc_largas[4]) ){
      argverb++;
      j=i;
      tmp=_argv[j];
    }
  }
  if( argverb==1 ){
    for(i=j; i>1; i--)
      _argv[i]=_argv[i-1];
    _argv[1]=tmp;
  }
  if( argverb > 1)
    error_std("-v", _("too many 'verbose'."), 0);
  // Sin opciones termina
  if( _argc == (1+argverb) )
    error_std(_("null argument"), _("no such options."), 1);
  // Segundo argumento debe ser una opcion
  if( _argc == (2+argverb) && obtopc(_argv[1+argverb]) < 5 )
    error_std(_("null argument"), _("no such argument."), 1);
  // Verificacion de opcion '--outfile'
  if( obtopc(_argv[1+argverb]) == ARCHIVO ){
    argout=1;
    if( _argc < (4+argverb) )
      error_std(_("null argument"), _("no such argument."), 1);
  }
  // Si hay muchos argumentos termina
  if( _argc > (3+argout+argverb) )
    error_std(_argv[_argc-1], _("too many arguments."), 0);
  // Ultimo argumento un numero
  if( _argc == (3+argout+argverb) && !verificar_numero(_argv[_argc-1]) )
    error_std(_argv[_argc-1], _("NaN, politropic index is not a number."), 0);
  return 1+argverb;
}

/* Mensaje de ayuda */
static void usotxt(void){
  static char const * const msj_help[]={
    N_(" Options:"),
    N_("  -r, --raiz              Evaluates the root of the Lane-Emden function of index n"),
    N_("  -d, --derivada          Evaluates derivative of the Lane-Emden function of index n in"),
    N_("                          the root"),
    N_("  -a, --all               Numerical value of the root, derivative, density ratio,"),
    N_("                          dimensionless mass and dimensionless pressure"),
    N_("  -o dat, --outfile dat   Print numerical solution of the function and your derivative in"),
    N_("                          the file 'dat'"),
    N_("  -v, --verbose           Show progress of the solution"),
    N_("  -V, --version           Show version"),
    N_("  -h, --help              Show this help"),
    0};
  char const * const *p = msj_help;
  cout << " " << programa << _(" - Numerical solution of the equation Lane-Emden") << endl;
  cout << _("Usage: ") << programa << _(" [options] <politropic index>") << endl;
  while( *p )
    cout << _(*p++) << endl;
  exit(EXIT_SUCCESS);
}

/* Mensaje de version */
static void versiontxt(void){
  static char const * const msj_licencia[] = {
    "Copyright (C) 2012-2015 Miguel Molina",
    N_("License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>"),
    N_("This program comes with ABSOLUTELY NO WARRANTY. This is free software, and you"),
    N_("are welcome to redistribute it under certain conditions."),
    0};
  char const * const *p = msj_licencia;
#ifdef _AUTOCONF_
  cout << PACKAGE_NAME << " " << PACKAGE_VERSION << endl;
#endif
  while( *p )
    cout << _(*p++) << endl;
  exit(EXIT_SUCCESS);
}

/**
 * @brief Programa principal de la aplicación CLI.
 * @param argc  Número de comandos en la línea de opciones.
 * @param argv  Comandos en la línea de opciones.
 * @return      Estado de salida del programa.
*/

/* Programa solvelen */
int main(int argc, char * argv[]){
  int arg, opn, verbose;
  double n, xi1n, dThetaxi1n; 
  double f[5];
  programa=basenameprog(argv[0]);
  // Soporte i18n
#ifdef ENABLE_NLS
  setlocale(LC_NUMERIC, "C");
  setlocale(LC_TIME, "");
  setlocale(LC_MONETARY, "");
  setlocale(LC_CTYPE, "");
  setlocale(LC_COLLATE, "");
  setlocale(LC_MESSAGES, "");
  textdomain(PACKAGE);
  bindtextdomain(PACKAGE, LOCALEDIR);
#endif /* ENABLE_NLS */
  arg=verificar_arg_prog(argc, argv);
  verbose=0;
  if( arg > 1 )
    verbose=1;
  n=atof(argv[argc-1]);
  if( n >= 5 || n <= -1 )
    error_std(argv[argc-1], _("politropic index must to be -1 < n < 5."), 0);
  else if( n<0 )
    cerr << _("Warning: '") << n << _("'; negative index, result with low accuracy.") << endl;
  // Ejecucion de opcion
  opn=obtopc( argv[arg] );
  switch( opn ){
  case RAIZ:
    Theta(n, f, verbose);
    xi1n=RaizXi1n(f);
    cout << setprecision(12) << xi1n << endl;
    break;
  case DERIVADA:
    Theta(n, f, verbose);
    xi1n=RaizXi1n(f);
    dThetaxi1n=DerivadaThetaXi1n(xi1n, f);
    cout << setprecision(12) << dThetaxi1n << endl;
    break;
  case TODO:
    // prog (-v) -a n | prog -a (-v) n
    Theta(n, f, verbose);
    xi1n=RaizXi1n(f);
    dThetaxi1n=DerivadaThetaXi1n(xi1n, f);
    cout << "      n             Xo           y'|Xo";
    cout << "    (-3y'/x)|Xo    (-x^2y')|Xo             Wn" << endl;
    // Indice n
    cout.setf(ios::right);
    cout << setw(7) << n << " ";
    cout.unsetf(ios::right);
    cout.setf( ios::scientific ); 
    // Raiz Xi1n
    cout << setprecision(8) << xi1n << " ";
    // Derivada D[Theta](xi1n)
    cout << setprecision(8) << dThetaxi1n << " ";
    // Razon de densidades -3 D[Theta](xi1n)/xi1n
    cout << setprecision(8) << -3.0*dThetaxi1n/xi1n << " ";
    // Masa adimensional -xi1n^2 D[Theta](xi1n)
    cout << setprecision(8) << -xi1n*xi1n*dThetaxi1n << " ";
    // Presion adimensoinal 1/(4 pi (n+1)) D[Theta](xi1n)^{-2}
    cout << setprecision(8) << 1.0/(4.0*npi*(n+1.0)*dThetaxi1n*dThetaxi1n) << endl;
    cout.unsetf(ios::scientific);
    break;
  case ARCHIVO:
    DatosTheta(n, argv[arg+1], verbose);
    break;
  case VERSNPROG:
    versiontxt();
    break;
  case AYUDA:
    usotxt();
    break;
  }
  return 0;
}
