/*
 * nsolveLEn.cpp - Funciones numericas para resolver la funcion de Lane-Emden de indice n
 *
 *    Copyright (C) 2012 - 2015  Miguel Molina
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/**
 * @file nsolveLEn.cpp
 * @brief Funciones numéricas para resolver la función de Lane-Emden
 * de índice n
 */

#include<iostream>
#include<fstream>
#include<iomanip>
#include<cmath>
#include"LaneEmden.h"
#include"biseccion.h"

using namespace std;

/// @cond INTERNAL

/* Numero total de iteraciones en MRK (pronostico!) */
double NTiteraciones(double x){
  double aux;
  aux=x/5.0;
  return (9798.0+4316.0*(1.0-exp(-0.407*pow(-log(1.0-fabs(aux)), 1.26))))/(1.0-aux);
}

/* Archivo de datos de la funcion Lane-Emden de indice n */
void DatosTheta(double n0, char * filedat, int verb){
  int progress, ctrl;
  double t, iter, Niter;
  LaneEmden solve;
  ofstream sale(filedat);
  progress=0;
  ctrl=0;
  iter=0;
  Niter=NTiteraciones(n0);
  t=0.0;
  solve.MatrizButcher();
  solve.inicio(n0);
  solve.updateRn(t);
  while( solve.flag() ){
    // h(imp_t)=1.0e-3
    if( !int(fmod(iter, 4)) )
      sale << setprecision(15) << solve.imp_t() << " " << solve.imp_x() << " " << solve.imp_y() << endl;
    solve.RungeKuttaS(dt);
    t+=dt;
    iter++;
    progress=int(100.0*iter/Niter);
    if( (progress % 5) == 0 && progress != ctrl && verb == 1 ){
      ctrl=progress;
      cout << progress << " %" << endl;
    }
    solve.updateRn(t);
  }
  // Sloppy Bug!
  if( progress < 100  && verb == 1 )
    cout << "100 %" << endl;
  sale.close();
}

/* Funcion Lane-Emden de indice n */
void Theta(double n0, double *v, int verb){
  int progress, ctrl;
  double t, tmp, iter, Niter;
  double a[6]={0, 0, 0, 0, 0, 0};
  LaneEmden solve;
  progress=0;
  ctrl=0;
  iter=0;
  Niter=NTiteraciones(n0);
  t=0.0;
  solve.MatrizButcher();
  solve.inicio(n0);
  solve.updateRn(t);
  while( solve.flag() ){
    a[0]=a[1];   a[1]=a[2];   a[2]=a[3];   a[3]=a[4];   a[4]=a[5];   a[5]=solve.imp_x();
    solve.RungeKuttaS(dt);
    t+=dt;
    iter++;
    progress=int(100.0*iter/Niter);
    if( (progress % 5) == 0 && progress != ctrl && verb == 1 ){
      ctrl=progress;
      cout << progress << " %" << endl;
    }
    solve.updateRn(t);
  }
  // Sloppy Bug!
  if( progress < 100  && verb == 1 )
    cout << "100 %" << endl;
  // Interpolacion de Lagrange de 4 puntos, error ~ 2.1504*h^4*D^4[f](x0)
  // fant2=a[0], fant1=a[1], f0=a[2], fpos1=a[3], fpos2=a[4], fpos3=a[5], fpos4=tmp
  tmp=(16.0*a[5]-15.0*a[4]+5.0*a[2]-a[0])/5.0;
  // fant1=v[0], f0=v[1], fpos1=v[2], fpos2=v[3]
  v[0]=a[3];   v[1]=a[4];   v[2]=a[5];   v[3]=tmp;
  v[4]=t-2.0*dt;
}

/* Interpolacion de funcion Lane-Emden de indice n */
double InterpolacionTheta(double t, double *fn){
  double p, t0, fp;
  double A[4];
  t0=fn[4];
  p=(t-t0)/dt;
  A[0]=-p*(p-1.0)*(p-2.0)/6.0;
  A[1]=(p*p-1.0)*(p-2.0)/2.0;
  A[2]=-p*(p+1.0)*(p-2.0)/2.0;
  A[3]=p*(p*p-1.0)/6.0;
  // Interpolacion de Lagrange de 4 puntos, error ~ 0.042*h^4*D^4[f](x0)
  fp=A[0]*fn[0]+A[1]*fn[1]+A[2]*fn[2]+A[3]*fn[3];
  return fp;
}

/* Primera raiz de la funcion Lane-Emden de indice n */
double RaizXi1n(double *fn){
  double raiz, epsXi, epsTheta;
  epsXi=1.0e-14;
  epsTheta=1.0e-12;
  raiz=biseccion(fn, epsXi, epsTheta);
  return raiz;
}

/* Derivada de la funcion Lane-Emden de indice n evaluada en la raiz */
double DerivadaThetaXi1n(double xi1n0, double *fn){
  double p, t0, dfp;
  double A[4];
  t0=fn[4];
  p=(xi1n0-t0)/dt;
  A[0]=-(3.0*p*p-6.0*p+2.0)/6.0;
  A[1]=(3.0*p*p-4.0*p-1.0)/2.0;
  A[2]=-(3.0*p*p-2.0*p-2.0)/2.0;
  A[3]=(3.0*p*p-1.0)/6.0;
  // Interpolacion de Lagrange de 4 puntos para 1ra derivada
  dfp=(A[0]*fn[0]+A[1]*fn[1]+A[2]*fn[2]+A[3]*fn[3])/dt;
  return dfp;
}

/// @endcond
