/*
 * nsolveLEn.h - Definicion de funciones numericas
 *
 *    Copyright (C) 2012 - 2015  Miguel Molina
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/**
 * @file nsolveLEn.h
 * @brief Declaración de funciones numéricas
 * @details Definición de diversas funciones numéricas que involucran
 * el uso de \f$\theta_n(t)\f$.
 */

#ifndef __NSOLVELEN_H_
#define __NSOLVELEN_H_

extern const double dt;              /**< Variable global para el paso de integración */
/**
 * @brief Número de iteraciones
 * @param x  Índice de la función de Lane-Emden
 * @return Entrega el número de iteraciones según índice.
 * @details Devuelve una estimación del número total de iteraciones
 * usadas en el método Runge-Kutta para encontrar la raíz
 * \f$\xi_{1n}\f$. La función que calcula el valor estimado, es
 * determinada para un paso de integración <i>h</i>=2.5e-4. Para un
 * paso de inegración <i>h</i> diferente la función puede cambiar.
 */
double NTiteraciones(double x);
/**
 * @brief Datos \f$\theta_n(t)\f$
 * @param n0  Índice \f$n\f$
 * @param filedat  Nombre archivo de datos
 * @param verb  Bandera modo verboso
 * @details Crea archivo de datos de la función Lane-Emden de índice
 * \f$n\f$; el archivo contiene el valor numérico de \f$\xi\f$,
 * \f$\theta_n(\xi)\f$ y \f$\theta'(\xi)\f$ en el intervalo
 * \f$[0, \xi_{1n})\f$.
 */
void DatosTheta(double n0, char * filedat, int verb);
/**
 * @brief Función extrapolación \f$\theta_n\f$
 * @param n0  Índice \f$n\f$
 * @param *v  Vector de extrapolación
 * @param verb  Bandera modo verboso
 * @details Extrapolación de la función Lane-Emden de índice \f$n\f$
 * para calcular un punto falso \f$\xi_N\f$, tal que 
 * \f$\theta_n(\xi_N)<0\f$. Cuando el índice \f$n\f$ es un número entero
 * \f$\theta_n(\xi_N)\f$ es un valor (real) aproximado de la funcion
 * exacta, pero cuando \f$n\f$ es un número real no entero, 
 * \f$\theta_n(\xi_N)\f$ es un valor (real) ficticio o falso de la
 * función exacta, puesto que la evaluación de
 * <i>pow</i>(\f$\xi\f$,\f$n\f$) en la ecuación diferencial con
 * \f$\xi<0\f$ arroja un número complejo.
 * La extrapolación usa interpolación de Lagrange de 4 puntos con
 * abcisas igualmente espaciadas de longitud <i>2h</i>.
 */
void Theta(double n0, double *v, int verb);
/**
 * @brief Función interpolación \f$\theta_n\f$
 * @param t  Variable independiente
 * @param fn Vector de interpolación
 * @details Interpolación de la función Lane-Emden de índice \f$n\f$
 * para calcular \f$\theta_n(\xi)\f$ en el intervalo
 * \f$[\xi_{N-1},\xi_N]\f$. Se asume que \f$\theta_n(\xi_{N-1})\f$
 * y \f$\theta_n(\xi_N)\f$ son de signos opuestos, lo cual asegura la
 * existencia de una raíz dentro del intervalo.
 * La interpolación usa interpolacion de Lagrange de 4 puntos con
 * abcisas igualmente espaciadas de longitud <i>h</i>.
 */
double InterpolacionTheta(double t, double *fn);
/**
 * @brief Primera raíz de la función de Lane-Emden de índice \f$n\f$
 * @param fn  Vector con interpolación
 * @return Entrega primera raíz \f$\xi_{1n}\f$ de la función de
 * Lane-Emden de índice \f$n\f$.
 * @details Cálculo aproximado de la primera raíz de la función de
 * Lane-Emden de índice \f$n\f$. La raíz es hallada a través del
 * método de la bisección, usando la función de interpolación para el
 * último intervalo de la solución numérica.
 */
double RaizXi1n(double *fn);
/**
 * @brief Derivada de la función de Lane-Emden de índice \f$n\f$
 * @param xi1n0  Primera raíz función de Lane-Emden
 * @param fn  Vector con interpolación
 * @return Entrega el valor numérico de la primera derivada de la
 * función Lane-Emden evaluada en \f$\xi_{1n}\f$.
 * @details  Valor numérico de la derivada de la función Lane-Emden de
 * índice \f$n\f$ evaluada en la primera raíz. El valor numérico es
 * calculado usando interpolación de Lagrange de 4 puntos para la
 * primera derivada con abcisas igualmente espaciadas de longitud
 * <i>h</i>.
 */
double DerivadaThetaXi1n(double xi1n0, double *fn);

#endif /* __NSOLVELEN_H_ */
